"""Templatetags for the ``cmsplugin_blog_categories`` app."""
from django import template
from django.db.models import Count

from cmsplugin_blog_categories.models import Category, EntryCategory


register = template.Library()


@register.assignment_tag
def get_category(entry=None):
    if entry:
        try:
            return EntryCategory.objects.get(entry=entry).category
        except EntryCategory.DoesNotExist:
            pass
    return None


@register.inclusion_tag(
    'cmsplugin_blog_categories/category_links_snippet.html',
    takes_context=True)
def render_category_links(context, exclude_empty=False):
    """Renders a list of all categories in the database."""
    qs = Category.objects.annotate(num_posts=Count('entry_categories'))
    if exclude_empty:
        qs = qs.exclude(num_posts=0)
    context.update({'categories': qs, })
    return context



"""CMS apphook for the ``cmsplugin_blog_categories`` app."""
from django.utils.translation import ugettext_lazy as _

from cms.app_base import CMSApp
from cms.apphook_pool import apphook_pool


class BlogCategoriesApphook(CMSApp):
    name = _("Blog Categories Apphook")
    urls = ["cmsplugin_blog_categories.urls"]


apphook_pool.register(BlogCategoriesApphook)



"""CMS Plugins for the ``cmsplugin_blog_categories`` app."""
from django.utils.translation import ugettext as _

from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool

from cmsplugin_blog_categories.models import CategoryPlugin


class CMSCategoryPlugin(CMSPluginBase):
    model = CategoryPlugin
    name = _('Blog Category')
    render_template = 'cmsplugin_blog_categories/category_plugin.html'

    def render(self, context, instance, placeholder):
        entries = [category.entry for category
                   in instance.category.entry_categories.all()
                   if category.entry.is_published]
        context.update({
            'category': instance.category,
            'category_entries': entries,
            'placeholder': placeholder,
        })
        return context

plugin_pool.register_plugin(CMSCategoryPlugin)



"""
Utilities for creating test objects related to the
``cmsplugin_blog_categories`` app.

"""
from cmsplugin_blog.models import Entry, EntryTitle
import factory

from cmsplugin_blog_categories import models


class CategoryFactory(factory.Factory):
    """Factory for the ``Category`` model."""
    FACTORY_FOR = models.Category

    slug = factory.LazyAttribute(lambda a: 'category-{0}'.format(a))


class CategoryTitleFactoryBase(factory.Factory):
    """Base factory for factories for ``CategoryTitle`` models."""
    FACTORY_FOR = models.CategoryTitle

    category = factory.SubFactory(CategoryFactory)


class CategoryTitleENFactory(CategoryTitleFactoryBase):
    """Factory for english ``CategoryTitle`` objects."""
    title = 'Category Title'
    language = 'en-us'


class CategoryTitleCNFactory(CategoryTitleFactoryBase):
    """Factory for chinese ``CategoryTitle`` objects."""
    title = unichr(34900)
    language = 'zh-cn'


class CategoryPluginFactory(factory.Factory):
    """Base factory for factories for ``CategoryPlugin`` models."""
    FACTORY_FOR = models.CategoryPlugin

    category = factory.SubFactory(CategoryFactory)


class EntryFactory(factory.Factory):
    """Base factory for factories for ``Entry`` models."""
    FACTORY_FOR = Entry

    is_published = True


class EntryTitleFactory(factory.Factory):
    """Base factory for factories for ``EntryTitle`` models."""
    FACTORY_FOR = EntryTitle

    entry = factory.SubFactory(EntryFactory)


class EntryCategoryFactory(factory.Factory):
    """Base factory for factories for ``EntryCategory`` models."""
    FACTORY_FOR = models.EntryCategory

    category = factory.SubFactory(CategoryFactory)
    entry = factory.SubFactory(EntryFactory)



# flake8: noqa
# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'CategoryPlugin'
        db.create_table('cmsplugin_categoryplugin', (
            ('cmsplugin_ptr', self.gf('django.db.models.fields.related.OneToOneField')(to=orm['cms.CMSPlugin'], unique=True, primary_key=True)),
            ('category', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['cmsplugin_blog_categories.Category'])),
        ))
        db.send_create_signal('cmsplugin_blog_categories', ['CategoryPlugin'])


    def backwards(self, orm):
        # Deleting model 'CategoryPlugin'
        db.delete_table('cmsplugin_categoryplugin')


    models = {
        'cms.cmsplugin': {
            'Meta': {'object_name': 'CMSPlugin'},
            'changed_date': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'creation_date': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2013, 2, 15, 0, 0)'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language': ('django.db.models.fields.CharField', [], {'max_length': '15', 'db_index': 'True'}),
            'level': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'lft': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['cms.CMSPlugin']", 'null': 'True', 'blank': 'True'}),
            'placeholder': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['cms.Placeholder']", 'null': 'True'}),
            'plugin_type': ('django.db.models.fields.CharField', [], {'max_length': '50', 'db_index': 'True'}),
            'position': ('django.db.models.fields.PositiveSmallIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'rght': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'tree_id': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'})
        },
        'cms.placeholder': {
            'Meta': {'object_name': 'Placeholder'},
            'default_width': ('django.db.models.fields.PositiveSmallIntegerField', [], {'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'slot': ('django.db.models.fields.CharField', [], {'max_length': '50', 'db_index': 'True'})
        },
        'cmsplugin_blog.entry': {
            'Meta': {'ordering': "('-pub_date',)", 'object_name': 'Entry'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_published': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'placeholders': ('djangocms_utils.fields.M2MPlaceholderField', [], {'to': "orm['cms.Placeholder']", 'symmetrical': 'False'}),
            'pub_date': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'tags': ('tagging.fields.TagField', [], {})
        },
        'cmsplugin_blog_categories.category': {
            'Meta': {'object_name': 'Category'},
            'creation_date': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '512'})
        },
        'cmsplugin_blog_categories.categoryplugin': {
            'Meta': {'object_name': 'CategoryPlugin', 'db_table': "'cmsplugin_categoryplugin'", '_ormbases': ['cms.CMSPlugin']},
            'category': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['cmsplugin_blog_categories.Category']"}),
            'cmsplugin_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['cms.CMSPlugin']", 'unique': 'True', 'primary_key': 'True'})
        },
        'cmsplugin_blog_categories.categorytitle': {
            'Meta': {'object_name': 'CategoryTitle'},
            'category': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['cmsplugin_blog_categories.Category']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language': ('django.db.models.fields.CharField', [], {'max_length': '5'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '256'})
        },
        'cmsplugin_blog_categories.entrycategory': {
            'Meta': {'unique_together': "(('entry', 'category'),)", 'object_name': 'EntryCategory'},
            'category': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'entry_categories'", 'to': "orm['cmsplugin_blog_categories.Category']"}),
            'entry': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'categories'", 'to': "orm['cmsplugin_blog.Entry']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        }
    }

    complete_apps = ['cmsplugin_blog_categories']



"""Tests for models of the ``cmsplugin_blog_categories``` application."""
from django.test import TestCase

from cmsplugin_blog_categories.cms_plugins import CMSCategoryPlugin
from cmsplugin_blog_categories.tests.factories import CategoryPluginFactory


class CMSCategoryPluginTestCase(TestCase):
    """Tests for the ``CMSCategoryPlugin`` cmsplugin."""
    longMessage = True

    def setUp(self):
        self.plugin = CategoryPluginFactory()
        self.cmsplugin = CMSCategoryPlugin()

    def test_render(self):
        self.assertEqual(
            self.cmsplugin.render(context={}, instance=self.plugin,
                                  placeholder=None),
            {'category': self.plugin.category, 'category_entries': [],
             'placeholder': None})



"""Admin classes for the ``cmsplugin_blog_categories`` app."""
from django.contrib import admin
from django.utils.translation import get_language
from django.utils.translation import ugettext_lazy as _

from cmsplugin_blog.admin import EntryAdmin
from simple_translation.admin import TranslationAdmin
from simple_translation.utils import get_preferred_translation_from_lang

from cmsplugin_blog_categories.models import Category, EntryCategory


class EntryCategoryInline(admin.TabularInline):
    model = EntryCategory
    max_num = 1


class EntryCategoryAdmin(admin.ModelAdmin):
    list_display = ['entry_title', 'category_title', ]

    def category_title(self, obj):
        """Returns the best available translation for the category title."""
        return obj.category.get_translation().title
    category_title.short_description = _('Category')

    def entry_title(self, obj):
        """Returns the best available translation for the entry title."""
        lang = get_language()
        entry_title = get_preferred_translation_from_lang(obj.entry, lang)
        return entry_title.title
    entry_title.short_description = _('Entry title')


class CategoryAdmin(TranslationAdmin):
    list_display = ['title', 'languages', ]

    def title(self, obj):
        """Returns the best available translation for the catrgory title."""
        return obj.get_translation().title
    title.short_description = _('Title')


# Enhance original EntryAdmin
EntryAdmin.inlines = EntryAdmin.inlines[:] + [EntryCategoryInline]

# Register our own admins
admin.site.register(Category, CategoryAdmin)
admin.site.register(EntryCategory, EntryCategoryAdmin)



"""URLs of the ``cmsplugin_blog_categories`` app. """
from django.conf.urls.defaults import include, patterns, url

from cmsplugin_blog_categories.views import CategoryListView


urlpatterns = patterns(
    '',
    url(r'^category/(?P<category>[^/]*)/',
        CategoryListView.as_view(),
        name='blog_archive_category',),

    url(r'^', include('cmsplugin_blog.urls')),
)



"""Views of the ``cmsplugin_blog_categories`` app."""
from django.views.generic import ListView

from cmsplugin_blog_categories.models import Category


class CategoryListView(ListView):
    template_name = 'cmsplugin_blog_categories/entry_archive_category.html'
    context_object_name = 'entries'

    def dispatch(self, request, *args, **kwargs):
        self.category = Category.objects.get(slug=kwargs.get('category'))
        return super(CategoryListView, self).dispatch(
            request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        ctx = super(CategoryListView, self).get_context_data(**kwargs)
        ctx.update({'category': self.category, })
        return ctx

    def get_queryset(self):
        return [category.entry for category
                in self.category.entry_categories.all()
                if category.entry.is_published]



"""Models for the ``cmsplugin_blog_categories`` app."""
from django.db import models
from django.utils.translation import get_language
from django.utils.translation import ugettext_lazy as _

from cms.models import CMSPlugin
from simple_translation.utils import get_preferred_translation_from_lang


class Category(models.Model):
    """
    A blog ``Entry`` can belong to one category.

    :slug: The slug for this category. The slug will be the same for all
      languages.

    """
    creation_date = models.DateTimeField(auto_now_add=True)

    slug = models.SlugField(
        max_length=512,
        verbose_name=_('Slug'),
    )

    def __unicode__(self):
        return self.get_title()

    @models.permalink
    def get_absolute_url(self):
        return ('blog_archive_category', (), {'category': self.slug, })

    def get_title(self):
        return self.get_translation().title

    def get_translation(self):
        lang = get_language()
        return get_preferred_translation_from_lang(self, lang)


class CategoryTitle(models.Model):
    title = models.CharField(
        max_length=256,
        verbose_name=_('Title'),
    )

    # Needed by simple-translation
    category = models.ForeignKey(Category)
    language = models.CharField(max_length=5)


class EntryCategory(models.Model):
    """Model that extends the ``Entry`` model of cmsplugin-blog."""
    entry = models.ForeignKey(
        'cmsplugin_blog.Entry',
        related_name='categories',
        verbose_name=_('Entry'),
    )

    category = models.ForeignKey(
        Category,
        related_name='entry_categories',
        verbose_name=_('Category'),
    )

    class Meta:
        unique_together = ('entry', 'category', )


class CategoryPlugin(CMSPlugin):
    """Plugin, which renders entries belonging to one category."""
    category = models.ForeignKey(
        Category,
        verbose_name=_('Category'),
    )



"""Tests for models of the ``cmsplugin_blog_categories``` application."""
from django.test import TestCase

from cmsplugin_blog_categories.tests.factories import (
    CategoryFactory,
    CategoryTitleCNFactory,
    CategoryTitleENFactory,
    EntryCategoryFactory,
    EntryFactory,
)


class CategoryTestCase(TestCase):
    """Tests for the ``Category`` model class."""
    longMessage = True

    def setUp(self):
        self.obj = CategoryFactory()

    def test_model(self):
        self.assertTrue(self.obj.pk)

    def test_get_title(self):
        self.assertEqual(self.obj.get_title(), 'None')
        CategoryTitleENFactory(category=self.obj)
        self.assertEqual(self.obj.get_title(), 'Category Title')


class CategoryTitleTestCase(TestCase):
    """Tests for the ``CategoryTitle`` model class."""
    longMessage = True

    def setUp(self):
        self.obj_en = CategoryTitleENFactory()
        self.obj_cn = CategoryTitleCNFactory()

    def test_model(self):
        self.assertTrue(self.obj_en.pk)
        self.assertTrue(self.obj_cn.pk)


class EntryTestCase(TestCase):
    """Tests for the ``Entry`` model class."""
    longMessage = True

    def setUp(self):
        self.obj = EntryFactory()

    def test_model(self):
        self.assertTrue(self.obj.pk)


class EntryCategoryTestCase(TestCase):
    """Tests for the ``EntryCategory`` model class."""
    longMessage = True

    def setUp(self):
        self.obj = EntryCategoryFactory()

    def test_model(self):
        self.assertTrue(self.obj.pk)



"""Registering translated models for the ``cmsplugin_blog_categories`` app."""
from simple_translation.translation_pool import translation_pool

from cmsplugin_blog_categories.models import Category, CategoryTitle


translation_pool.register_translation(Category, CategoryTitle)



"""Tests for tags of the ``cmsplugin_blog_categories``` application."""
from django.test import TestCase

from cmsplugin_blog_categories.templatetags import (
    cmsplugin_blog_categories_tags as tags,
)
from cmsplugin_blog_categories.tests.factories import (
    CategoryFactory,
    EntryFactory,
    EntryCategoryFactory,
)


class GetCategoryTestCase(TestCase):
    """Tests for the ``get_category`` tag."""
    longMessage = True

    def setUp(self):
        self.entry = EntryFactory()
        self.category = EntryCategoryFactory()

    def test_tag(self):
        self.assertFalse(tags.get_category())
        self.assertFalse(tags.get_category(self.entry))
        self.assertEqual(tags.get_category(self.category.entry),
                         self.category.category)


class RenderCategoryLinksTestCase(TestCase):
    """Tests for the ``render_category_links`` tag."""
    longMessage = True

    def setUp(self):
        self.entry_category = EntryCategoryFactory()
        self.category = CategoryFactory()

    def test_tag(self):
        self.assertEqual(tags.render_category_links({}).get('categories')[0],
                         self.entry_category.category)
        self.assertEqual(
            tags.render_category_links({}, True).get('categories').count(), 1)



"""Tests for the views of the ``cmsplugin_blog_categories`` app."""
from django.test import TestCase

from django_libs.tests.mixins import ViewTestMixin

from cmsplugin_blog_categories.tests.factories import (
    CategoryTitleENFactory,
    EntryFactory,
)


class CategoryListViewTestCase(ViewTestMixin, TestCase):
    """Tests for the ``CategoryListView`` generic view class."""
    def setUp(self):
        self.category_title = CategoryTitleENFactory()
        self.category = self.category_title.category
        self.entry = EntryFactory()

    def get_view_name(self):
        return 'blog_archive_category'

    def get_view_kwargs(self):
        return {'category': self.category.slug}

    def test_view(self):
        self.should_be_callable_when_anonymous()
